# AniGif Package written in pure tcl
#
# anigif.tcl v0.9 2001-01-11 (c) 2001 Ryan Casey
#
# AniGif is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# AniGif is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# For a copy of the GNU General Public License, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
#
# ###############################  USAGE  #################################
#
#  ::anigif::anigif FILENAME WIDGET DELAY INDEX
#    FILENAME: appropriate path and file to use for the animated gif
#    WIDGET:   a label widget to place the animated gif into
#    DELAY:    how long to wait before loading next image (Default: 100ms)
#    INDEX:    what image to begin on (first image is 0) (Default: 0)
#
#  ::anigif::stop WIDGET
#  ::anigif::restart WIDGET INDEX
#    INDEX:    defaults to next index in loop
#  ::anigif::destroy WIDGET

namespace eval anigif {

  proc anigif2 {w name list {delay 100} {idx 0}} {
#puts "anigif2 \{ $w $name $list $delay $idx \}"
    if { ![winfo exists $w] } {
      eval [linsert $list 0 image delete]
    } else {
      if { $idx >= [llength $list] } { set idx 0 }
      $w itemconfigure $name -image [lindex $list $idx]
      update
      set ::anigif_${w}_${name}(idx) [incr idx]
      set ::anigif_${w}_${name}(asdf) "::anigif::anigif2 $w $name [list $list]"
      set ::anigif_${w}_${name}(loop) [after $delay "[set ::anigif_${w}_${name}(asdf)] $delay $idx"]
    }
  }

  proc anigif {fnam w name {delay 100} {idx 0}} {
#puts "anigif \{ $fnam $w $name $delay $idx \}"
    set n 0
    set images {}
    while {![catch "image create photo pic$n$w$name \
      -file ${fnam} \
      -format \{gif -index $n\}"]} {
        lappend images pic$n$w$name
        incr n
    }
    set ::anigif_${w}_${name}(delay) $delay
    anigif2 $w $name $images $delay $idx
  }

  proc stop {w name} {
    catch {
      after cancel [set ::anigif_${w}_${name}(loop)]
    }
  }

  proc restart {w name {idx -1}} {
    if {$idx == -1} {
      set idx [set ::anigif_${w}_${name}(idx)]
    }
    catch {
      ::anigif::stop $w $name
      eval "[set ::anigif_${w}_${name}(asdf)] [set ::anigif_${w}_${name}(delay)] $idx"
    }
  }

  proc destroy {w} {
    catch {
      ::anigif::stop $w $name
#      set wlength [string length $w]
      foreach imagename [image names] {
        if {[string equal [string range $imagename [string first "." $imagename] end] $w]} {
          image delete $imagename
        }
      }
     unset ::anigif_${w}_${name}
    }
  }

}

package provide anigif 0.9
