#!/bin/sh
# Port Monitor 0.5, 20010219
# Copyright 2001 Ryan Casey
# Provion, LLC
#
# the next line restarts using wish\
exec wish "$0" "$@" 

lappend auto_path ./bwidget
lappend auto_path ./anigif_bw
package require anigif
package require BWidget

#################################
# USER DEFINED PROCEDURES
#

proc {load_monitor} {address port name} {
#
  .l insert end ${address}_${port} -image $::img0 -text "$name"
  .l itemconfigure ${address}_${port} -fill #0F0
  ::anigif::anigif curley_sm_blk.gif .l ${address}_${port}
  ::anigif::stop   .l ${address}_${port}
  update
  return
}

proc isWindows {} {
  if {"$::tcl_platform(platform)" == "windows"} {
    return 1
  } else {
    return 0
  }
}

proc _firstWritable { socket address port name {flag G} } {

  fileevent $socket writable {}

# 4500000 microseconds per iteration if it fails to resolve
  if {[catch {fconfigure $socket -peername} peerinfo]} {
    switch $flag {
      G {
          .l itemconfigure ${address}_${port} -fill #FF0
          after $::dis_int [list monitor $address $port $name Y]
        }
      Y {
          .l itemconfigure ${address}_${port} -fill #F00
          if { [isWindows] } {
            exec $::env(COMSPEC) /c net send $::notify "$name (${address}:${port}) DOWN!" &
          }
          after $::dis_int [list monitor $address $port $name R]
        }
      R {
          after $::dis_int [list monitor $address $port $name R]
        }
    }
  } else {
    switch $flag {
      Y {
          .l itemconfigure ${address}_${port} -fill #0F0
        }
      R {
          .l itemconfigure ${address}_${port} -fill #0F0
          if { [isWindows] } {
            exec $::env(COMSPEC) /c net send $::notify "$name (${address}:${port}) UP!" &
          }
        }
    }
    after $::con_int [list monitor $address $port $name G]
  }

  close $socket
  ::anigif::stop .l ${address}_${port}

  return
}


proc {monitor} {address port name {flag G}} {

  ::anigif::restart .l ${address}_${port}
  set test [catch {socket -async $address $port} ::channel($address,$port)]

  after 1000 [fileevent $::channel($address,$port) writable [list _firstWritable $::channel($address,$port) $address $port $name $flag]]

  return
}

proc {stop_monitors} {} {
  foreach afterid [after info] {
    after cancel $afterid
  }
  foreach {name address port} $::monitors {
    catch {fileevent $::channel($address,$port) writable {}}
  }  
}

proc {start_monitors} {} {
  ::anigif::restart .l noaddress_noport
  set i 1
  foreach {name address port} $::monitors {
    after [expr {500 * $i}] [list monitor $address $port $name]
    incr i
  }
}

proc {main} {argc argv} {
#
  load_monitor noaddress noport "Monitor"
  .l insert end sepaddress_sepport -text "[string repeat "-" 12]"
  .l itemconfigure sepaddress_sepport -fill #0F0

  foreach {name address port} $::monitors {
    load_monitor $address $port $name
  }
  start_monitors
}

proc init {argc argv} {
#
  if { [isWindows] } {
# Changes icon to PortMon icon if on Windows
    set icofile ./icon/portmon.ico 
    if {[file exists $icofile]} {
      set tmpfile ./portmon.ico
      set fin [open $icofile r] 
      fconfigure $fin -translation binary 
      set fout [open $tmpfile w] 
      fconfigure $fout -translation binary 
      puts -nonewline $fout [read $fin] 
      close $fin 
      close $fout 
      set ico [winico createfrom $tmpfile] 
      winico setwindow . $ico
    }
  }

  set ::img0 [image create photo -file curley_sm_blk.gif]

  set ::monitors {}
  set fp [open "./portmon.ini" r]
  set iniInfo [split [read $fp [file size "./portmon.ini"]] \n]
  close $fp
  foreach line $iniInfo {
    if {"$line" != ""} {
      set line [string trim $line]
      set portstart [expr {[string last " " $line] + 1}]
      set port [string range $line $portstart end]
      set line [string trim [string range $line 0 [expr {$portstart - 1}]]]
      set addressstart [expr {[string last " " $line] + 1}]
      set address [string range $line $addressstart end]
      set line [string trim [string range $line 0 [expr {$addressstart - 1}]]]
      set name $line
      lappend ::monitors $name $address $port
    }
  }
  
  set ::con_int 60000
  set ::dis_int 10000
  set ::notify RCASEY

  return
}

wm title . "Port Monitor 0.6"
frame .f
button .f.butStart -command start_monitors -text Start
button .f.butStop -command stop_monitors -text Stop
button .f.butExit -command exit -text Exit

ListBox .l -background #000 

pack .f -fill both -expand no
pack .f.butStart -in .f -anchor center -expand 0 -fill none -padx 1 -pady 1 -side left 
pack .f.butStop -in .f -anchor center -expand 0 -fill none -padx 1 -pady 1 -side left 
pack .f.butExit -in .f -anchor center -expand 0 -fill none -padx 1 -pady 1 -side right

pack .l -fill both -expand yes
update

init $argc $argv
main $argc $argv
